@@ -1,14 +1,14 @@
-from pyfirmata import *
-from boards import BOARDS
+from .pyfirmata import *
+from .boards import BOARDS
+
+__version__ = '1.0.0'  # Don't forget to change in setup.py!
 
-__version__ = '0.9.5'  # Don't forget to change in setup.py!
 
 # shortcut classes
 
-
 class Arduino(Board):
     """
-    A board that wil set itself up as a normal Arduino.
+    A board that will set itself up as a normal Arduino.
     """
     def __init__(self, *args, **kwargs):
         args = list(args)
@@ -16,12 +16,12 @@ class Arduino(Board):
         super(Arduino, self).__init__(*args, **kwargs)
 
     def __str__(self):
-        return 'Arduino %s on %s' % (self.name, self.sp.port)
+        return "Arduino {0.name} on {0.sp.port}".format(self)
 
 
 class ArduinoMega(Board):
     """
-    A board that wil set itself up as an Arduino Mega.
+    A board that will set itself up as an Arduino Mega.
     """
     def __init__(self, *args, **kwargs):
         args = list(args)
@@ -29,4 +29,4 @@ class ArduinoMega(Board):
         super(ArduinoMega, self).__init__(*args, **kwargs)
 
     def __str__(self):
-        return 'Arduino Mega %s on %s' % (self.name, self.sp.port)
+        return "Arduino Mega {0.name} on {0.sp.port}".format(self)
