@@ -1,10 +1,13 @@
-import serial
+from __future__ import division, unicode_literals
+
 import inspect
 import time
-import itertools
-from util import two_byte_iter_to_str, to_two_bytes
 
-# Message command bytes - straight from Firmata.h
+import serial
+
+from .util import to_two_bytes, two_byte_iter_to_str
+
+# Message command bytes (0x80(128) to 0xFF(255)) - straight from Firmata.h
 DIGITAL_MESSAGE = 0x90      # send data for a digital pin
 ANALOG_MESSAGE = 0xE0       # send data for an analog pin (or PWM)
 DIGITAL_PULSE = 0x91        # SysEx command to send a digital pulse
@@ -36,7 +39,7 @@ SYSEX_REALTIME = 0x7F       # MIDI Reserved for realtime messages
 
 # Pin modes.
 # except from UNAVAILABLE taken from Firmata.h
-UNAVAILABLE = -1 
+UNAVAILABLE = -1
 INPUT = 0          # as defined in wiring.h
 OUTPUT = 1         # as defined in wiring.h
 ANALOG = 2         # analog pin in analogInput mode
@@ -50,19 +53,21 @@ DIGITAL = OUTPUT   # same as OUTPUT below
 # Time to wait after initializing serial, used in Board.__init__
 BOARD_SETUP_WAIT_TIME = 5
 
+
 class PinAlreadyTakenError(Exception):
     pass
 
+
 class InvalidPinDefError(Exception):
     pass
-    
+
+
 class NoInputWarning(RuntimeWarning):
     pass
-    
+
+
 class Board(object):
-    """
-    Base class for any board
-    """
+    """The Base class for any board."""
     firmata_version = None
     firmware = None
     firmware_version = None
@@ -70,11 +75,11 @@ class Board(object):
     _command = None
     _stored_data = []
     _parsing_sysex = False
-    
+
     def __init__(self, port, layout, baudrate=57600, name=None):
         self.sp = serial.Serial(port, baudrate)
         # Allow 5 secs for Arduino's auto-reset to happen
-        # Alas, Firmata blinks it's version before printing it to serial
+        # Alas, Firmata blinks its version before printing it to serial
         # For 2.3, even 5 seconds might not be enough.
         # TODO Find a more reliable way to wait until the board is ready
         self.pass_time(BOARD_SETUP_WAIT_TIME)
@@ -85,26 +90,26 @@ class Board(object):
         # Iterate over the first messages to get firmware data
         while self.bytes_available():
             self.iterate()
-        # TODO Test whether we got a firmware name and version, otherwise there 
+        # TODO Test whether we got a firmware name and version, otherwise there
         # probably isn't any Firmata installed
-        
+
     def __str__(self):
-        return "Board %s on %s" % (self.name, self.sp.port)
-        
+        return "Board{0.name} on {0.sp.port}".format(self)
+
     def __del__(self):
-        ''' 
+        """
         The connection with the a board can get messed up when a script is
         closed without calling board.exit() (which closes the serial
         connection). Therefore also do it here and hope it helps.
-        '''
+        """
         self.exit()
-        
+
     def send_as_two_bytes(self, val):
-        self.sp.write(chr(val % 128) + chr(val >> 7))
+        self.sp.write(bytearray([val % 128, val >> 7]))
 
     def setup_layout(self, board_layout):
         """
-        Setup the Pin instances based on the given board-layout. Maybe it will
+        Setup the Pin instances based on the given board layout. Maybe it will
         be possible to do this automatically in the future, by polling the
         board for its type.
         """
@@ -115,15 +120,15 @@ class Board(object):
 
         self.digital = []
         self.digital_ports = []
-        for i in xrange(0, len(board_layout['digital']), 8):
-            num_pins = len(board_layout['digital'][i:i+8])
-            port_number = i / 8
+        for i in range(0, len(board_layout['digital']), 8):
+            num_pins = len(board_layout['digital'][i:i + 8])
+            port_number = int(i / 8)
             self.digital_ports.append(Port(self, port_number, num_pins))
 
         # Allow to access the Pin instances directly
         for port in self.digital_ports:
             self.digital += port.pins
-        
+
         # Setup PWM pins
         for i in board_layout['pwm']:
             self.digital[i].PWM_CAPABLE = True
@@ -133,37 +138,41 @@ class Board(object):
             self.digital[i].mode = UNAVAILABLE
 
         # Create a dictionary of 'taken' pins. Used by the get_pin method
-        self.taken = { 'analog' : dict(map(lambda p: (p.pin_number, False), self.analog)),
-                       'digital' : dict(map(lambda p: (p.pin_number, False), self.digital)) }
+        self.taken = {'analog': dict(map(lambda p: (p.pin_number, False), self.analog)),
+                      'digital': dict(map(lambda p: (p.pin_number, False), self.digital))}
 
         # Setup default handlers for standard incoming commands
         self.add_cmd_handler(ANALOG_MESSAGE, self._handle_analog_message)
         self.add_cmd_handler(DIGITAL_MESSAGE, self._handle_digital_message)
         self.add_cmd_handler(REPORT_VERSION, self._handle_report_version)
         self.add_cmd_handler(REPORT_FIRMWARE, self._handle_report_firmware)
-    
+
     def add_cmd_handler(self, cmd, func):
-        """ 
-        Adds a command handler for a command.
-        """
+        """Adds a command handler for a command."""
         len_args = len(inspect.getargspec(func)[0])
+
         def add_meta(f):
             def decorator(*args, **kwargs):
                 f(*args, **kwargs)
-            decorator.bytes_needed = len_args - 1 # exclude self
+            decorator.bytes_needed = len_args - 1  # exclude self
             decorator.__name__ = f.__name__
             return decorator
         func = add_meta(func)
         self._command_handlers[cmd] = func
-        
+
     def get_pin(self, pin_def):
         """
         Returns the activated pin given by the pin definition.
         May raise an ``InvalidPinDefError`` or a ``PinAlreadyTakenError``.
-        
-        :arg pin_def: Pin definition as described in TODO,
+
+        :arg pin_def: Pin definition as described below,
             but without the arduino name. So for example ``a:1:i``.
-        
+
+        'a' analog pin     Pin number   'i' for input
+        'd' digital pin    Pin number   'o' for output
+                                        'p' for pwm (Pulse-width modulation)
+
+        All seperated by ``:``.
         """
         if type(pin_def) == list:
             bits = pin_def
@@ -173,11 +182,11 @@ class Board(object):
         part = getattr(self, a_d)
         pin_nr = int(bits[1])
         if pin_nr >= len(part):
-            raise InvalidPinDefError('Invalid pin definition: %s at position 3 on %s' % (pin_def, self.name))
-        if getattr(part[pin_nr], 'mode', None)  == UNAVAILABLE:
-            raise InvalidPinDefError('Invalid pin definition: UNAVAILABLE pin %s at position on %s' % (pin_def, self.name))
+            raise InvalidPinDefError('Invalid pin definition: {0} at position 3 on {1}'.format(pin_def, self.name))
+        if getattr(part[pin_nr], 'mode', None) == UNAVAILABLE:
+            raise InvalidPinDefError('Invalid pin definition: UNAVAILABLE pin {0} at position on {1}'.format(pin_def, self.name))
         if self.taken[a_d][pin_nr]:
-            raise PinAlreadyTakenError('%s pin %s is already taken on %s' % (a_d, bits[1], self.name))
+            raise PinAlreadyTakenError('{0} pin {1} is already taken on {2}'.format(a_d, bits[1], self.name))
         # ok, should be available
         pin = part[pin_nr]
         self.taken[a_d][pin_nr] = True
@@ -186,49 +195,38 @@ class Board(object):
                 pin.mode = PWM
             elif bits[2] == 's':
                 pin.mode = SERVO
-            elif bits[2] is not 'o':
+            elif bits[2] != 'o':
                 pin.mode = INPUT
         else:
             pin.enable_reporting()
         return pin
-        
+
     def pass_time(self, t):
-        """ 
-        Non-blocking time-out for ``t`` seconds.
-        """
+        """Non-blocking time-out for ``t`` seconds."""
         cont = time.time() + t
         while time.time() < cont:
             time.sleep(0)
-            
-    def send_sysex(self, sysex_cmd, data=[]):
+
+    def send_sysex(self, sysex_cmd, data):
         """
         Sends a SysEx msg.
-        
+
         :arg sysex_cmd: A sysex command byte
-        :arg data: A list of 7-bit bytes of arbitrary data (bytes may be 
-            already converted to chr's)
+        : arg data: a bytearray of 7-bit bytes of arbitrary data
         """
-        self.sp.write(chr(START_SYSEX))
-        self.sp.write(chr(sysex_cmd))
-        for byte in data:
-            try:
-                byte = chr(byte)
-            except TypeError:
-                pass # byte is already a chr
-            except ValueError:
-                raise ValueError('Sysex data can be 7-bit bytes only. '
-                    'Consider using utils.to_two_bytes for bigger bytes.')
-            self.sp.write(byte)
-        self.sp.write(chr(END_SYSEX))
-        
+        msg = bytearray([START_SYSEX, sysex_cmd])
+        msg.extend(data)
+        msg.append(END_SYSEX)
+        self.sp.write(msg)
+
     def bytes_available(self):
         return self.sp.inWaiting()
 
     def iterate(self):
-        """ 
+        """
         Reads and handles data from the microcontroller over the serial port.
-        This method should be called in a main loop, or in an
-        :class:`Iterator` instance to keep this boards pin values up to date
+        This method should be called in a main loop or in an :class:`Iterator`
+        instance to keep this boards pin values up to date.
         """
         byte = self.sp.read()
         if not byte:
@@ -266,40 +264,42 @@ class Board(object):
             handler(*received_data)
         except ValueError:
             pass
-            
+
     def get_firmata_version(self):
         """
-        Returns a version tuple (major, mino) for the firmata firmware on the
+        Returns a version tuple (major, minor) for the firmata firmware on the
         board.
         """
         return self.firmata_version
-        
+
     def servo_config(self, pin, min_pulse=544, max_pulse=2400, angle=0):
         """
         Configure a pin as servo with min_pulse, max_pulse and first angle.
         ``min_pulse`` and ``max_pulse`` default to the arduino defaults.
         """
         if pin > len(self.digital) or self.digital[pin].mode == UNAVAILABLE:
-            raise IOError("Pin %s is not a valid servo pin")
-        data = itertools.chain([pin], to_two_bytes(min_pulse),
-                                        to_two_bytes(max_pulse))
+            raise IOError("Pin {0} is not a valid servo pin".format(pin))
+
+        data = bytearray([pin])
+        data += to_two_bytes(min_pulse)
+        data += to_two_bytes(max_pulse)
         self.send_sysex(SERVO_CONFIG, data)
-        
+
         # set pin._mode to SERVO so that it sends analog messages
         # don't set pin.mode as that calls this method
         self.digital[pin]._mode = SERVO
         self.digital[pin].write(angle)
-        
+
     def exit(self):
-        """ Call this to exit cleanly. """
+        """Call this to exit cleanly."""
         # First detach all servo's, otherwise it somehow doesn't want to close...
-        # FIXME
-        for pin in self.digital:
-            if pin.mode == SERVO:
-                pin.mode = OUTPUT
+        if hasattr(self, 'digital'):
+            for pin in self.digital:
+                if pin.mode == SERVO:
+                    pin.mode = OUTPUT
         if hasattr(self, 'sp'):
             self.sp.close()
-        
+
     # Command handlers
     def _handle_analog_message(self, pin_nr, lsb, msb):
         value = round(float((msb << 7) + lsb) / 1023, 4)
@@ -313,7 +313,7 @@ class Board(object):
     def _handle_digital_message(self, port_nr, lsb, msb):
         """
         Digital messages always go by the whole port. This means we have a
-        bitmask wich we update the port.
+        bitmask which we update the port.
         """
         mask = (msb << 7) + lsb
         try:
@@ -323,70 +323,70 @@ class Board(object):
 
     def _handle_report_version(self, major, minor):
         self.firmata_version = (major, minor)
-        
+
     def _handle_report_firmware(self, *data):
         major = data[0]
         minor = data[1]
         self.firmware_version = (major, minor)
         self.firmware = two_byte_iter_to_str(data[2:])
 
+
 class Port(object):
-    """ An 8-bit port on the board """
+    """An 8-bit port on the board."""
     def __init__(self, board, port_number, num_pins=8):
         self.board = board
         self.port_number = port_number
         self.reporting = False
-        
+
         self.pins = []
         for i in range(num_pins):
             pin_nr = i + self.port_number * 8
             self.pins.append(Pin(self.board, pin_nr, type=DIGITAL, port=self))
-            
+
     def __str__(self):
-        return "Digital Port %i on %s" % (self.port_number, self.board)
-        
+        return "Digital Port {0.port_number} on {0.board}".format(self)
+
     def enable_reporting(self):
-        """ Enable reporting of values for the whole port """
+        """Enable reporting of values for the whole port."""
         self.reporting = True
-        msg = chr(REPORT_DIGITAL + self.port_number)
-        msg += chr(1)
+        msg = bytearray([REPORT_DIGITAL + self.port_number, 1])
         self.board.sp.write(msg)
+
         for pin in self.pins:
             if pin.mode == INPUT:
-                pin.reporting = True # TODO Shouldn't this happen at the pin?
-        
+                pin.reporting = True  # TODO Shouldn't this happen at the pin?
+
     def disable_reporting(self):
-        """ Disable the reporting of the port """
+        """Disable the reporting of the port."""
         self.reporting = False
-        msg = chr(REPORT_DIGITAL + self.port_number)
-        msg += chr(0)
+        msg = bytearray([REPORT_DIGITAL + self.port_number, 0])
         self.board.sp.write(msg)
-                
+
     def write(self):
-        """Set the output pins of the port to the correct state"""
+        """Set the output pins of the port to the correct state."""
         mask = 0
         for pin in self.pins:
             if pin.mode == OUTPUT:
                 if pin.value == 1:
                     pin_nr = pin.pin_number - self.port_number * 8
-                    mask |= 1 << pin_nr
-        msg = chr(DIGITAL_MESSAGE + self.port_number)
-        msg += chr(mask % 128)
-        msg += chr(mask >> 7)
+                    mask |= 1 << int(pin_nr)
+#        print("type mask", type(mask))
+#        print("type self.portnumber", type(self.port_number))
+#        print("type pinnr", type(pin_nr))
+        msg = bytearray([DIGITAL_MESSAGE + self.port_number, mask % 128, mask >> 7])
         self.board.sp.write(msg)
-        
+
     def _update(self, mask):
-        """
-        Update the values for the pins marked as input with the mask.
-        """
+        """Update the values for the pins marked as input with the mask."""
         if self.reporting:
             for pin in self.pins:
                 if pin.mode is INPUT:
                     pin_nr = pin.pin_number - self.port_number * 8
                     pin.value = (mask & (1 << pin_nr)) > 0
 
+
 class Pin(object):
-    """ A Pin representation """
+    """A Pin representation"""
     def __init__(self, board, pin_number, type=ANALOG, port=None):
         self.board = board
         self.pin_number = pin_number
@@ -396,108 +396,100 @@ class Pin(object):
         self._mode = (type == DIGITAL and OUTPUT or INPUT)
         self.reporting = False
         self.value = None
-        
+
     def __str__(self):
-        type = {ANALOG : 'Analog', DIGITAL : 'Digital'}[self.type]
-        return "%s pin %d" % (type, self.pin_number)
+        type = {ANALOG: 'Analog', DIGITAL: 'Digital'}[self.type]
+        return "{0} pin {1}".format(type, self.pin_number)
 
     def _set_mode(self, mode):
         if mode is UNAVAILABLE:
             self._mode = UNAVAILABLE
             return
         if self._mode is UNAVAILABLE:
-            raise IOError("%s can not be used through Firmata" % self)
+            raise IOError("{0} can not be used through Firmata".format(self))
         if mode is PWM and not self.PWM_CAPABLE:
-            raise IOError("%s does not have PWM capabilities" % self)
+            raise IOError("{0} does not have PWM capabilities".format(self))
         if mode == SERVO:
             if self.type != DIGITAL:
-                raise IOError("Only digital pins can drive servos! %s is not"
-                    "digital" % self)
+                raise IOError("Only digital pins can drive servos! {0} is not"
+                    "digital".format(self))
             self._mode = SERVO
             self.board.servo_config(self.pin_number)
             return
-        
+
         # Set mode with SET_PIN_MODE message
         self._mode = mode
-        command = chr(SET_PIN_MODE)
-        command += chr(self.pin_number)
-        command += chr(mode)
-        self.board.sp.write(command)
+        self.board.sp.write(bytearray([SET_PIN_MODE, self.pin_number, mode]))
         if mode == INPUT:
             self.enable_reporting()
-        
+
     def _get_mode(self):
         return self._mode
-        
+
     mode = property(_get_mode, _set_mode)
     """
     Mode of operation for the pin. Can be one of the pin modes: INPUT, OUTPUT,
-    ANALOG, PWM or SERVO (or UNAVAILABLE)
+    ANALOG, PWM. or SERVO (or UNAVAILABLE).
     """
-    
+
     def enable_reporting(self):
-        """ Set an input pin to report values """
+        """Set an input pin to report values."""
         if self.mode is not INPUT:
-            raise IOError, "%s is not an input and can therefore not report" % self
+            raise IOError("{0} is not an input and can therefore not report".format(self))
         if self.type == ANALOG:
             self.reporting = True
-            msg = chr(REPORT_ANALOG + self.pin_number)
-            msg += chr(1)
+            msg = bytearray([REPORT_ANALOG + self.pin_number, 1])
             self.board.sp.write(msg)
         else:
-            self.port.enable_reporting() # TODO This is not going to work for non-optimized boards like Mega
-        
+            self.port.enable_reporting()
+            # TODO This is not going to work for non-optimized boards like Mega
+
     def disable_reporting(self):
-        """ Disable the reporting of an input pin """
+        """Disable the reporting of an input pin."""
         if self.type == ANALOG:
             self.reporting = False
-            msg = chr(REPORT_ANALOG + self.pin_number)
-            msg += chr(0)
+            msg = bytearray([REPORT_ANALOG + self.pin_number, 0])
             self.board.sp.write(msg)
         else:
-            self.port.disable_reporting() # TODO This is not going to work for non-optimized boards like Mega
-    
+            self.port.disable_reporting()
+            # TODO This is not going to work for non-optimized boards like Mega
+
     def read(self):
         """
         Returns the output value of the pin. This value is updated by the
-        boards :meth:`Board.iterate` method. Value is alway in the range 0.0 - 1.0
+        boards :meth:`Board.iterate` method. Value is always in the range from
+        0.0 to 1.0.
         """
         if self.mode == UNAVAILABLE:
-            raise IOError, "Cannot read pin %s"% self.__str__()
+            raise IOError("Cannot read pin {0}".format(self.__str__()))
         return self.value
-        
+
     def write(self, value):
         """
         Output a voltage from the pin
 
         :arg value: Uses value as a boolean if the pin is in output mode, or
-            expects a float from 0 to 1 if the pin is in PWM mode. If the pin 
+            expects a float from 0 to 1 if the pin is in PWM mode. If the pin
             is in SERVO the value should be in degrees.
-        
+
         """
         if self.mode is UNAVAILABLE:
-            raise IOError, "%s can not be used through Firmata" % self
+            raise IOError("{0} can not be used through Firmata".format(self))
         if self.mode is INPUT:
-            raise IOError, "%s is set up as an INPUT and can therefore not be written to" % self
+            raise IOError("{0} is set up as an INPUT and can therefore not be written to".format(self))
         if value is not self.value:
             self.value = value
             if self.mode is OUTPUT:
                 if self.port:
                     self.port.write()
                 else:
-                    msg = chr(DIGITAL_MESSAGE)
-                    msg += chr(self.pin_number)
-                    msg += chr(value)
+                    msg = bytearray([DIGITAL_MESSAGE, self.pin_number, value])
                     self.board.sp.write(msg)
             elif self.mode is PWM:
                 value = int(round(value * 255))
-                msg = chr(ANALOG_MESSAGE + self.pin_number)
-                msg += chr(value % 128)
-                msg += chr(value >> 7)
+                msg = bytearray([ANALOG_MESSAGE + self.pin_number, value % 128, value >> 7])
                 self.board.sp.write(msg)
             elif self.mode is SERVO:
                 value = int(value)
-                msg = chr(ANALOG_MESSAGE + self.pin_number)
-                msg += chr(value % 128)
-                msg += chr(value >> 7)
-                self.board.sp.write(msg)
\ No newline at end of file
+                msg = bytearray([ANALOG_MESSAGE + self.pin_number, value % 128, value >> 7])
+                self.board.sp.write(msg)
